/**
 * Copyright (c) by Mohan Embar 2008. All Rights Reserved.
 * http://www.thisiscool.com/
 */
using System;
using System.Collections.Generic;
using System.Text;

using System.Windows.Forms;

using com.thisiscool.tinybasic;

namespace TinyBasicUI
{
class TinyBasicTextAreaManager
{
public TinyBasicTextAreaManager(TextBox text)
{
	this.text = text;
    text.KeyDown += new KeyEventHandler(this.OnKeyDown);
	
	tb = new TinyBasic(new MyConsoleIO(this));
    startTinyBasic();
    runTinyBasic();
}

public void Tick()
{
    runTinyBasic();
}
	
// private //
private TinyBasic tb;
private int lastReturnCode = 0;
private TextBox text;

private string current = null;
private int curpos;
private int endpos;

private void startTinyBasic()
{
	tb.StartTinyBasic(null);
}

private void runTinyBasic()
{
	if (current == null
			&& (lastReturnCode == TinyBasic.INTERP_NEEDCHAR || lastReturnCode == TinyBasic.INTERP_NEEDLINE))
		return;
		// we're not going to get any farther
	
	lastReturnCode = tb.RunTinyBasic(lastReturnCode);

	// Commit current the ending position.
	endpos = text.Text.Length;
}

private void OnKeyDown(object sender, KeyEventArgs e)
{
	string t = text.Text;
	int len = t.Length;

	text.Select(len,len);
    if (e.KeyCode==Keys.D && e.Control)
    {
        TinyBasic.Debugging ^= 1;
    }
    else if (e.KeyCode == Keys.Escape)
    {
        if (e.Control)
            TinyBasic.Debugging = 1;
        else
            tb.setBroken(true);
    }
    else if (e.KeyCode == Keys.Enter)
    {
        if (e.Modifiers == 0)
        {
            if (len < endpos)
                return;

            current = t.Substring(endpos, len - endpos) + "\r";
            curpos = 0;

            // Kick the interpreter again. We do this later because the
            // key hasn't been consumed yet.
            text.BeginInvoke(new MethodInvoker(runTinyBasic));
        }
    }
}

//////////////////////////////////////////////////////////////
// inner class MyConsoleIO

private class MyConsoleIO : IConsoleIO
{
public void print(string theMsg)
{
    if (theMsg == "\r")
        theMsg = "\r\n";
    else
	    theMsg = theMsg.Replace("\n", "\r\n");
	Append(theMsg);
}

public char read()
{
	if (manager.current == null)
		return '\0';
    else if (manager.curpos >= manager.current.Length)
	{
        manager.current = null;
		return '\0';
	}
    return manager.current[manager.curpos++];
}

public void screenChar(char ch)
{
	if (ch == '\n')
		Append("\r\n");
    else
	    Append(""+ch);
}

// internal //
internal MyConsoleIO(TinyBasicTextAreaManager manager)
{
    this.manager = manager;
}

// private //
private TinyBasicTextAreaManager manager;

private void Append(string s)
{
    string t = manager.text.Text;
	if (t.Length > 65536)
        manager.text.Text = "";
    manager.text.AppendText(s);
}
}

// end inner class MyConsoleIO
//////////////////////////////////////////////////////////////
}
}
