/**
 * Copyright (c) by Mohan Embar 2008. All Rights Reserved.
 * http://www.thisiscool.com/
 */
package // com.thisiscool.tinybasic
{
import mx.controls.TextArea;
import mx.controls.textClasses.TextRange;

import mx.events.FlexEvent;

import mx.core.Application;

import flash.utils.Timer;

import flash.events.KeyboardEvent;
import flash.events.TextEvent;   
import flash.events.TimerEvent;

public class TinyBasicTextAreaManager implements IConsoleIO
{
public function TinyBasicTextAreaManager(tarea:TextArea, tdebug:TextArea)
{
	this.tarea = tarea;
	this.tdebug = tdebug;
	
	tarea.addEventListener(KeyboardEvent.KEY_DOWN, onKeyTyped);
	tarea.addEventListener(FlexEvent.VALUE_COMMIT, onValueCommitted);

	this.tb = new TinyBasic(this);
	startTinyBasic();
	runTinyBasic();
}

// IConsoleIO overrides
public function screenChar(ch:int):void
{
	print(String.fromCharCode(ch));
}

public function print(theMsg:String):void
{
  toprint += theMsg;
}

public function read():int
{
  var result:int = 0;
	if (current!=null && curpos<current.length)
		result = current.charCodeAt(curpos++);
	else
	  current = null;
	  
	if (result == 0)
		wantInput = true;
	  
	return result;
}

// private //
private var tarea:TextArea;
private var tdebug:TextArea;
private var toprint:String = "";

private var tb:TinyBasic;
private var lastReturnCode:int = 0;

private var current:String = null;
private var curpos:int;
private var endpos:int;
private var wantInput:Boolean = false;


private function onValueCommitted(event:FlexEvent):void
{
	tarea.validateNow();
	tarea.verticalScrollPosition = tarea.maxVerticalScrollPosition;
}

private function postUpdate():void
{
	if (toprint.length > 0)
	{
		var curlen:int = tarea.text.length;
		var start:int = curlen;
		
		if (toprint.indexOf("\r") != -1)
			toprint = toprint.split("\r").join("\n");
		
		if (curlen > 65536)
			start = 0;
		var tr:TextRange = new TextRange(tarea, false, start, curlen);
		tr.text = toprint;
		toprint = "";
  }
  
  curlen = tarea.text.length;
  tarea.setSelection(curlen,curlen);
  endpos = curlen;
}

private function startTinyBasic():void
{
	tb.StartTinyBasic(null);
	
	// Set a timer to kick the interpreter to start a new timeslice.
	var timer:Timer = new Timer(250,0);
	timer.addEventListener(TimerEvent.TIMER, onTimer);
	timer.start();
}

private function runTinyBasic():void
{
	if (current == null
			&& (lastReturnCode == tb.INTERP_NEEDCHAR || lastReturnCode == tb.INTERP_NEEDLINE))
		return;
		// we're not going to get any farther
	
	wantInput = false;
	lastReturnCode = tb.RunTinyBasic(lastReturnCode);

	// Commit current the ending position.
  Application.application.callLater(postUpdate);
}

private function onTimer(event:TimerEvent):void
{
	runTinyBasic();
}

private function onKeyTyped(e:KeyboardEvent):void
{
	if (e.charCode==100 && e.ctrlKey)
		tb.toggleDebugging();
  else if (e.charCode == 27)
  	tb.setBroken(true);
  else if (e.charCode == 13)
  {
		var t:String = tarea.text;
		var curend:int = t.length;

		if (endpos > curend)
			endpos = 0;

		if (curend < endpos)
			return;

		current = t.substring(endpos,curend) + "\r";
		curpos = 0;

		// Kick the interpreter again.
		runTinyBasic();
  }
}
}
}