/**
 * Copyright (c) by Mohan Embar 2008. All Rights Reserved.
 * http://www.thisiscool.com/
 */
package com.thisiscool.tinybasic.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import com.thisiscool.tinybasic.IConsoleIO;
import com.thisiscool.tinybasic.TinyBasic;

public class TinyBasicTextAreaManager
{
public TinyBasicTextAreaManager(final JTextArea text)
{
	this.text = text;
	text.setText("Welcome to TinyBasic/J.\nBased on http://www.ittybittycomputers.com/IttyBitty/TinyBasic/TinyBasic.c by Tom Pittman.\nPorted by Mohan Embar: http://www.thisiscool.com/.\nUse [Esc] to break, [Ctrl+D] to toggle debug diagnostics.\n");
	int len = text.getText().length();
	text.select(len,len);
	
	text.addKeyListener(new KeyAdapter() {
		@Override
		public void keyTyped(KeyEvent e)
		{
			int len = text.getText().length();
			text.select(len,len);
			if (e.getKeyChar() == 4) // [Ctrl+D]
				TinyBasic.Debugging ^= 1;
			else if (e.getKeyChar() == '\033')
				tb.setBroken(true);
			else if (e.getKeyChar() == '\n')
			{
				if (e.getModifiers() == 0)
					onEnterPressed();
			}
		}});

	tb = new TinyBasic(new MyConsoleIO());
	SwingUtilities.invokeLater(new Runnable() {
		public final void run()
		{
			// Do this in the UI thread.
			startTinyBasic();
			runTinyBasic();
		}});
}
	
// private //
private TinyBasic tb;
private int lastReturnCode = 0;
private JTextArea text;

private String current = null;
private int curpos;
private int endpos;

private void startTinyBasic()
{
	tb.StartTinyBasic(null);
	
	// Set a timer to kick the interpreter to start a new timeslice.
	new Timer(250, new ActionListener() {
		public final void actionPerformed(ActionEvent arg0)
		{
			runTinyBasic();
		}}).start();
}

private void runTinyBasic()
{
	if (current == null
			&& (lastReturnCode == TinyBasic.INTERP_NEEDCHAR || lastReturnCode == TinyBasic.INTERP_NEEDLINE))
		return;
		// we're not going to get any farther
	
	lastReturnCode = tb.RunTinyBasic(lastReturnCode);

	// Commit current the ending position.
	endpos = text.getText().length();
}

private void onEnterPressed()
{
	String t = text.getText();
	int curend = t.length();
	if (curend <= endpos)
		return;
	
	current = t.substring(endpos,curend);
	curpos = 0;
	
	// Kick the interpreter again.
	runTinyBasic();
}

//////////////////////////////////////////////////////////////
// inner class MyConsoleIO

private class MyConsoleIO implements IConsoleIO
{
public void print(String theMsg)
{
	theMsg = theMsg.replace('\r', '\n');
	append(theMsg);
}

public char read()
{
	if (current == null)
		return '\0';
	else if (curpos >= current.length())
	{
		current = null;
		return '\0';
	}
	return current.charAt(curpos++);
}

public void screenChar(char ch)
{
	if (ch == '\r')
		ch = '\n';
	append(""+ch);
}

// private //
private void append(String s)
{
	String t = text.getText();
	if (t.length() > 65536)
		text.setText("");
	text.append(s);
}
}

// end inner class MyConsoleIO
//////////////////////////////////////////////////////////////
}
